@@ -4,6 +4,7 @@
  * \brief C error handling
  */
 #include <dmlc/thread_local.h>
+#include "xgboost/c_api.h"
 #include "./c_api_error.h"
 
 struct XGBAPIErrorEntry {
@@ -12,7 +13,7 @@ struct XGBAPIErrorEntry {
 
 using XGBAPIErrorStore = dmlc::ThreadLocalStore<XGBAPIErrorEntry>;
 
-const char *XGBGetLastError() {
+XGB_DLL const char *XGBGetLastError() {
   return XGBAPIErrorStore::Get()->last_error.c_str();
 }
 
